import matplotlib.pyplot as plt, random

#set line width
plt.rcParams['lines.linewidth'] = 4
#set font size for titles 
plt.rcParams['axes.titlesize'] = 18
#set font size for labels on axes
plt.rcParams['axes.labelsize'] = 18
#set size of numbers on x-axis
plt.rcParams['xtick.labelsize'] = 16
#set size of numbers on y-axis
plt.rcParams['ytick.labelsize'] = 16
#set size of ticks on x-axis
plt.rcParams['xtick.major.size'] = 7
#set size of ticks on y-axis
plt.rcParams['ytick.major.size'] = 7
#set size of markers
plt.rcParams['lines.markersize'] = 10

#global constants
TRADINGDAYS = 250  #number of trading days/year

class Stock(object):
    def __init__(self, ticker, volatility):
        self._volatility = volatility
        self._ticker = ticker
    def setPrice(self, price):
        self._price = price
        self._history = [price]
    def makeMove(self):
        if self._price <= 0:
            return 0
        opening = self._price
        baseMove = random.uniform(-self._volatility,
                                  self._volatility)
        self._price = max(0, self._price + baseMove)
        self._history.append(self._price)
        return 100*(self._price - opening)/opening
    def getHistory(self):
        return self._history
    def getTicker(self):
        return self._ticker
    
class Market(object):
    def __init__(self):
        self._stks = []
    def addStk(self, stk):
        self._stks.append(stk)
    def getStks(self):
        return self._stks.copy()
    
def getAnnRet(stk):
    for d in range(TRADINGDAYS):
        stk.makeMove()
    hist = stk.getHistory()
    return 100*((hist[-1] - hist[0])/hist[0])

def testMkt(numStks, vol):
    mkt = Market()
    for i in range(numStks):
        stk = Stock(str(i), vol)
        stk.setPrice(25)
        mkt.addStk(stk)
    returns = []
    for stk in mkt.getStks():
        returns.append(getAnnRet(stk))
    mean = sum(returns)/len(returns)
    median = sorted(returns)[len(returns)//2]
    plt.hist(returns, bins = 40)
    plt.title('Gross Return, vol = ' + str(vol) +\
              '\n mean = ' + str(round(mean, 2)) +\
              ', median = ' + str(round(median, 2)))
    plt.xlabel('Return (%)')
    plt.ylabel('Number of Stocks')
    
testMkt(100, 0)
plt.figure()
testMkt(100, 5)
