import matplotlib.pyplot as plt, random

#set line width
plt.rcParams['lines.linewidth'] = 4
#set font size for titles 
plt.rcParams['axes.titlesize'] = 18
#set font size for labels on axes
plt.rcParams['axes.labelsize'] = 18
#set size of numbers on x-axis
plt.rcParams['xtick.labelsize'] = 16
#set size of numbers on y-axis
plt.rcParams['ytick.labelsize'] = 16
#set size of ticks on x-axis
plt.rcParams['xtick.major.size'] = 7
plt.rcParams['ytick.major.size'] = 7
#set size of markers
plt.rcParams['lines.markersize'] = 10

#global constants
TRADINGDAYS = 250  #number of trading days/year

class Stock(object):
    def __init__(self, ticker, bias, volatility):
        self._bias = bias/100
        self._vol = volatility/100
        self._ticker = ticker
    def makeMove(self):
        opening = self._price
        if self._price <= 0:
            return 0
        baseMove = self._price*(random.gauss(self._bias,
                                             self._vol))
        self._price = max(0, self._price + baseMove)
        self._history.append(self._price)
        return 100*((self._price - opening)/opening)
    def setPrice(self, price):
        self._price = price
        self._history = [price]
    def getHistory(self):
        return self._history
    def getTicker(self):
        return self._ticker
    
class Market(object):
    def __init__(self):
        self._stks = []
    def addStk(self, stk):
        self._stks.append(stk)
    def getStks(self):
        return self._stks.copy()
    
def getAnnRet(stk):
    for d in range(TRADINGDAYS):
        stk.makeMove()
    hist = stk.getHistory()
    return 100*((hist[-1] - hist[0])/hist[0])
    
def testMkt(mkt, toPlot = True):
    returns = []
    for stk in mkt.getStks():
        returns.append(getAnnRet(stk))
    mean = sum(returns)/len(returns)
    median = sorted(returns)[len(returns)//2]
    if toPlot:
        plt.hist(returns, bins = 40)
        plt.title('Gross Return' +\
                  '\n mean = ' + str(round(mean, 2)) +\
                  ', median = ' + str(round(median, 2)))
        plt.xlabel('Return (%)')
        plt.ylabel('Number of Stocks')
    return mean, median

def makeMkt(numStks):
    mkt = Market()
    for i in range(numStks):
        vol = random.gauss(0, 2)
        stk = Stock(str(i), 0.026, vol)
        stk.setPrice(25)
        mkt.addStk(stk)
    return mkt

#testMkt(makeMkt(100))

import copy

def simMkt(numTrials):
    initMkt = makeMkt(100)
    means, medians = [], []
    for t in range(numTrials):
        mkt = copy.deepcopy(initMkt)
        mean, median = testMkt(mkt, False)
        means.append(mean)
        medians.append(median)
    plt.hist(means, range=(-5, 20))
    mean = round(sum(means)/len(means), 2)
    plt.title('Distribution of Means\n(mean of means = ' +\
                                      str(mean) + '%)')
    plt.xlabel('Gross Return (%)')
    plt.ylabel('Number of Trials')
    plt.figure()
    plt.hist(medians, range=(-5, 20))
    median = round(sum(medians)/len(medians), 2)
    plt.title('Distribution of Medians\n(mean of medians = ' +\
                                      str(median) + '%)')
    plt.xlabel('Gross Return (%)')
    plt.ylabel('Number of Trials')

simMkt(1000)